<?php
/**
 * This file is part of the SysCP project.
 * Copyright (c) 2003-2007 the SysCP Team (see authors).
 *
 * For the full copyright and license information, please view the COPYING
 * file that was distributed with this source code. You can also view the
 * COPYING file online at http://files.syscp.org/misc/COPYING.txt
 *
 * @copyright  (c) the authors
 * @author     Florian Lippert <flo@syscp.org>
 * @license    GPLv2 http://files.syscp.org/misc/COPYING.txt
 * @package    Panel
 * @version    $Id: admin_configfiles.php 1153 2007-01-31 20:21:59Z flo $
 */

	define('AREA', 'admin');

	/**
	 * Include our init.php, which manages Sessions, Language etc.
	 */
	require("./lib/init.php");


	$configfiles = Array
	(
		'freebsd_release' => Array
		(
			'label' => 'FreeBSD 6.2 (RELEASE)',
			'daemons' => Array
			(
				'apache' => Array
				(
					'label' => 'Apache Webserver (HTTP)',
					'commands' => Array
					(
						'echo -e "\\nInclude '.$settings['system']['apacheconf_directory'].$settings['system']['apacheconf_filename'].'" >> '.$settings['system']['apacheconf_directory'].'httpd.conf',
						'touch '.$settings['system']['apacheconf_directory'].$settings['system']['apacheconf_filename'],
						'mkdir -p '.$settings['system']['documentroot_prefix'],
						'mkdir -p '.$settings['system']['logfiles_directory']
					),
					'restart' => Array
					(
						'/usr/local/etc/rc.d/apache' . ( $settings['system']['apacheversion'] == 'apache2' ? '22' : '' ) . ' restart'
					)
				),
				'bind' => Array
				(
					'label' => 'Bind9 Nameserver (DNS)',
					'files' => Array
					(
						'etc_bind_default.zone' => '/var/named/etc/namedb/default.zone'
					),
					'commands' => Array
					(
						'echo "include \"/etc/namedb/syscp_bind.conf\";" >> /var/named/etc/namedb/named.conf',
						'touch '.$settings['system']['bindconf_directory'].'syscp_bind.conf'
					),
					'restart' => Array
					(
						'/usr/sbin/rndc reload'
					)
				),
				'dovecot' => Array
				(
					'label' => 'Dovecot (POP3/IMAP)',
					'files' => Array
					(
						'etc_dovecot_conf' => '/usr/local/etc/dovecot.conf',
						'etc_dovecot_sql' => '/usr/local/etc/dovecot-sql.conf'
					),
					'restart' => Array
					(
						'/usr/local/etc/rc.d/dovecot restart',
					)
				),
				'postfix' => Array
				(
					'label' => 'Postfix (MTA)',
					'files' => Array
					(
						'etc_postfix_main.cf' => '/usr/local/etc/postfix/main.cf',
						'etc_postfix_mysql-virtual_alias_maps.cf' => '/usr/local/etc/postfix/mysql-virtual_alias_maps.cf',
						'etc_postfix_mysql-virtual_mailbox_domains.cf' => '/usr/local/etc/postfix/mysql-virtual_mailbox_domains.cf',
						'etc_postfix_mysql-virtual_mailbox_maps.cf' => '/usr/local/etc/postfix/mysql-virtual_mailbox_maps.cf',
					),
					'commands' => Array
					(
						'mkdir -p '.$settings['system']['vmail_homedir'],
						'chown -R postfix:postfix '.$settings['system']['vmail_homedir']
					),
					'restart' => Array
					(
						'/usr/local/sbin/postfix reload'
					)
				),
				'proftpd' => Array
				(
					'label' => 'ProFTPd (FTP)',
					'files' => Array
					(
						'etc_proftpd.conf' => '/usr/local/etc/proftpd.conf'
					),
					'restart' => Array
					(
						'/usr/local/etc/rc.d/proftpd restart'
					)
				),
				'cron' => Array
				(
					'label' => 'Crond (cronscript)',
					'commands' => Array
					(
					'mkdir /usr/local/etc/syscpcron',
					'cp -Rp /usr/local/etc/php.ini-recommended /usr/local/etc/syscpcron/php.ini'
					),
					'files' => Array
					(
						'etc_cron.d_syscp' => 'crontab -e (As root)'
					)
				)
			)
		)
	);

	if( ($page == 'configfiles' || $page == 'overview') && $userinfo['change_serversettings'] == '1')
	{
		if(isset($_GET['distribution']) && $_GET['distribution']!='' && isset($configfiles[$_GET['distribution']]) && is_array($configfiles[$_GET['distribution']]) &&
		   isset($_GET['daemon']) && $_GET['daemon']!='' && isset($configfiles[$_GET['distribution']]['daemons'][$_GET['daemon']]) && is_array($configfiles[$_GET['distribution']]['daemons'][$_GET['daemon']]))
		{
			$distribution = $_GET['distribution'];
			$daemon = $_GET['daemon'];

			if(isset($configfiles[$distribution]['daemons'][$daemon]['commands']) && is_array($configfiles[$distribution]['daemons'][$daemon]['commands']))
			{
				$commands = implode("\n", $configfiles[$distribution]['daemons'][$daemon]['commands']);
			}
			else
			{
				$commands = '';
			}

			$replace_arr = Array
			(
				'<SQL_UNPRIVILEGED_USER>' => $sql['user'],
				'<SQL_UNPRIVILEGED_PASSWORD>' => 'MYSQL_PASSWORD',
				'<SQL_DB>' => $sql['db'],
				'<SQL_HOST>' => $sql['host'],
				'<ADMIN_EMAIL>' => $settings['panel']['adminmail'],
				'<SERVERNAME>' => $settings['system']['hostname'],
				'<SERVERIP>' => $settings['system']['ipaddress'],
				'<VIRTUAL_MAILBOX_BASE>' => $settings['system']['vmail_homedir'],
				'<VIRTUAL_UID_MAPS>' => $settings['system']['vmail_uid'],
				'<VIRTUAL_GID_MAPS>' => $settings['system']['vmail_gid']
			);
			$files = '';
			if(isset($configfiles[$distribution]['daemons'][$daemon]['files']) && is_array($configfiles[$distribution]['daemons'][$daemon]['files']))
			{
				while(list($filename, $realname) = each($configfiles[$distribution]['daemons'][$daemon]['files']))
				{
					$file_content = file_get_contents('./templates/misc/configfiles/'.$distribution.'/'.$daemon.'/'.$filename);
					$file_content = strtr($file_content, $replace_arr);
					$file_content = htmlspecialchars($file_content);
					$numbrows = count(explode("\n", $file_content));
					eval("\$files.=\"".getTemplate("configfiles/configfiles_file")."\";");
				}
			}

			if(isset($configfiles[$distribution]['daemons'][$daemon]['restart']) && is_array($configfiles[$distribution]['daemons'][$daemon]['restart']))
			{
				$restart = implode("\n", $configfiles[$distribution]['daemons'][$daemon]['restart']);
			}
			else
			{
				$restart = '';
			}

			eval("echo \"".getTemplate("configfiles/configfiles")."\";");
		}
		else
		{
			$distributions = '';
			while (list($distribution_name, $distribution_details) = each($configfiles))
			{
				$daemons = '';
				while(list($daemon_name, $daemon_details) = each($distribution_details['daemons']))
				{
					eval("\$daemons.=\"".getTemplate("configfiles/choose_daemon")."\";");
				}
				eval("\$distributions.=\"".getTemplate("configfiles/choose_distribution")."\";");
			}
			eval("echo \"".getTemplate("configfiles/choose")."\";");
		}
	}

?>
